/* 
Gramatica 11: 'gr11.pl'
*/

use_module(library(lists)).
ana(A,E):-
	ej(A,B),
	a_lexic(B,C),
	a_sint(C,D),
	a_sem(D,E),
	gestion_orden(E).

a_sint(A,B):-
	oracion(B,A,[]).

a_sem([X:_],X).
a_sem([Head|Mods],Sem):-
	a_sem_s(Mods,Sems),
	componer(Head,Sems,Sem).
	
a_sem_s([],[]).
a_sem_s([Mod|Mods],[Sem|Sems]):-
	a_sem(Mod,Sem),
	a_sem_s(Mods,Sems).

componer(oracion,[X],X).
componer(declarativa,[[A,B],_,A],[declarativa,B]).
componer(interrogativa,[_,_,[A,B]],[interrogativa,A,B]).
componer(sn,[Base],[Base,[]]).
componer(sn,[X,[Y,Z]],[U,[R|Z]]):- 
	R=..[X,Y,U].
componer(sn_bas,[X],X).
componer(sn_bas,[_,Y],Y).
componer(sn_mod,[X],X).
componer(sp,[_,Y],Y).

gestion_orden([declarativa,R]):-
	declarar(R).
gestion_orden([interrogativa,Que,Restricc]):-
	responder(Que,Restricc).
gestion_orden(_):-
	print('sin efectos'),nl.

responder(Que,Restricc):-
	ejecutar_restricciones(Restricc),
	print('la respuesta es '),
	print(Que),nl.
responder(_,_):-
	print('no lo se').

ejecutar_restricciones([]).
ejecutar_restricciones([Restr|Restrs]):-
	ejecutar_restriccion(Restr),
	ejecutar_restricciones(Restrs).
ejecutar_restriccion(R):-
	clause(R,_),
	R.

declarar([]).
declarar([Dec|Decs]):-
	declarar_1(Dec),
	declarar(Decs).
declarar_1(Dec):-
	modificar(Dec).

modificar(Dec):-
	retract(Dec),
	fail.
modificar(Dec):-
	assertz(Dec).

oracion([oracion,X]) --> interrogativa(X).
oracion([oracion,X]) --> declarativa(X).
declarativa([declarativa,A,B,C]) --> sn(A), v_ser(B), sn_bas(C).
interrogativa([interrogativa,A,B,C]) --> p_int(A), v_ser(B), sn(C), sig_int(_).
sn([sn,X]) --> sn_bas(X).
sn([sn,A,B]) --> sn_bas(A), sn_mod(B).
sn_bas([sn_bas,X]) --> np(X).
sn_bas([sn_bas,X]) --> num(X).
sn_bas([sn_bas,A,B]) --> det(A), n(B).
sn_mod([sn_mod,X]) --> sp(X).
sp([sp,A,B])--> prep(A), sn(B).

p_int([W]) --> [W],{es_p_int(W)}.
v_ser([W]) -->[W],{es_v_ser(W)}.
det([W]) --> [W],{es_det(W)}.
n([W]) --> [W],{es_n(W)}.
prep([W]) --> [W],{es_prep(W)}.
num([W]) --> [W],{es_num(W)}.
np([W]) -->[W],{es_np(W)}.
sig_int([W]) --> [W],{es_sig_int(W)}.

es_p_int(_:X):-
	member(p_int,X).
es_v_ser(_:X):-
	member(v_ser,X).
es_det(_:X):-
	member(det,X).
es_n(_:X):-
	member(n,X).
es_prep(_:X):-
	member(prep,X).
es_np(_:X):-
	member(np,X).
es_sig_int(_:X):-
	member(sig_int,X).
es_num(_:X):-
	member(num,X).

